/*---------------------------------------------------------------------------------
 bbBroamTunes (2006 kazmix [kazmix@kazmix.com])
 ----------------------------------------------------------------------------------
 bbBroamTunes is a plugin for Blackbox for Windows. 
 --------------------------------------------------------------------------------*/

#ifndef _WIN32_WINNT
    #define _WIN32_WINNT 0x0500
#endif

#include "BBApi.h"
#ifndef _ATL_CSTRING_EXPLICIT_CONSTRUCTORS
#define _ATL_CSTRING_EXPLICIT_CONSTRUCTORS
#endif
#include <atlbase.h>
#include <atlstr.h>
#include <atlcom.h>
#include "iTunesCOMInterface.h"
#include "iTunesCOMInterface_i.c"

//---------------------------------------------------------------------------------

extern "C"
{
	DLL_EXPORT int beginPlugin(HINSTANCE hMainInstance);
	DLL_EXPORT void endPlugin(HINSTANCE hMainInstance);
	DLL_EXPORT LPCSTR pluginInfo(int x);
}

//---------------------------------------------------------------------------------

static HWND     hPluginWnd = NULL;
static int      bb_messages[] = {BB_BROADCAST, 0};
const char broam[] = "@bbBroamTunes";
const char szAppName[] = "bbBroamTunes";
CComPtr<IiTunes> iTunes;

//---------------------------------------------------------------------------------

LRESULT CALLBACK PluginProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    if (message == BB_BROADCAST)
    {
        if (!strnicmp(broam, (char*)(lParam), strlen(broam))){
		char *param;
		param = ((char*)(lParam)) + strlen(broam);
		while(*param==' ' || *param=='\t')
			param ++;
		
    		if(FAILED(iTunes.CoCreateInstance(CLSID_iTunesApp))){
           		MessageBox(NULL,"iTunes create error",szAppName,MB_OK);
            		return 1;
   		}
		VARIANT_BOOL vbool;
		ITPlayerState state;
		long volume;	
		if(!stricmp("Play",param)){
			iTunes->Play();
		}else if(!stricmp("Next",param)){
			iTunes->NextTrack();
		}else if(!stricmp("Prev",param)){
			iTunes->PreviousTrack();
		}else if(!stricmp("ToggleMute",param)){
			iTunes->get_Mute(&vbool);
			iTunes->put_Mute((vbool == VARIANT_TRUE)?VARIANT_FALSE:VARIANT_TRUE);
		}else if(!stricmp("TogglePlayPause",param)){
			iTunes->PlayPause();
		}else if(!stricmp("Stop",param)){
			iTunes->Stop();
		}else if(!stricmp("Fastforward",param)){
			iTunes->get_PlayerState(&state);
			if(state==ITPlayerStateFastForward){
				iTunes->Resume();
			}else{
				iTunes->FastForward();
			}
		}else if(!stricmp("Rewind",param)){
			iTunes->get_PlayerState(&state);
			if(state==ITPlayerStateRewind){
				iTunes->Resume();
			}else{
				iTunes->Rewind();
			}
		}else if(!stricmp("VolumeUp",param)){
			iTunes->get_SoundVolume(&volume);
			iTunes->put_SoundVolume(volume+5);
		}else if(!stricmp("VolumeDown",param)){
			iTunes->get_SoundVolume(&volume);
			iTunes->put_SoundVolume(volume-5);
		}
		iTunes.Release();
	}
        return 0;
    }
    return DefWindowProc(hwnd, message, wParam, lParam);
}

//---------------------------------------------------------------------------------

int beginPlugin(HINSTANCE h_instance)
{

    if(FAILED(::CoInitialize(NULL))){
	    MessageBox(NULL,"Initializing COM Error",szAppName,MB_OK);
	    return 1;
    }
	
    WNDCLASS wc;
    ZeroMemory(&wc, sizeof(wc));
    wc.lpfnWndProc = PluginProc;
    wc.hInstance = h_instance;
    wc.lpszClassName = szAppName;
    RegisterClass(&wc);

    hPluginWnd = CreateWindowEx(
        WS_EX_TOOLWINDOW,
        szAppName,
        NULL,
        WS_POPUP|WS_ICONIC,
        -4, -4, 0, 0,
        HWND_MESSAGE,
        NULL,
        h_instance,
        NULL
    );

    SendMessage(GetBBWnd(), BB_REGISTERMESSAGE, (WPARAM)hPluginWnd, (LPARAM)bb_messages);
    return 0;
}

//---------------------------------------------------------------------------------

void endPlugin(HINSTANCE h_instance)
{
	::CoUninitialize();
	DestroyWindow(hPluginWnd);
	SendMessage(GetBBWnd(), BB_UNREGISTERMESSAGE, (WPARAM)hPluginWnd, (LPARAM)bb_messages);
	UnregisterClass(szAppName, h_instance);
}

//---------------------------------------------------------------------------------
LPCSTR pluginInfo(int x)
{
	switch (x)
        {
		case 1: return "0.11";
		case 2: return szAppName;
		case 3: return "kazmix";
		case 4: return "2006/11/22";
		case 5: return "http://www.kazmix.com/cside.php";
		case 6: return "kazmix@kazmix.com";
		default: return "bbBroamTunes 0.11";
     
	}
}
